/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narQuat.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/// @cond DEV
/**
 *  @file
 *  @author Satoru Osako
 */

#ifndef NAR_QUAT_H__
#define NAR_QUAT_H__

#include "narConfig.h"
#include "narMath.h"

namespace mw { namespace nar
{
    /**
     *  NH[^jI
     */
    struct Quat_st
    {
        Quat_st()
        {}
        
        Quat_st( const Vec3F_st & cr_Xa, const Vec3F_st & cr_Ya, const Vec3F_st & cr_Za )
        {
            SetFromAxises( cr_Xa, cr_Ya, cr_Za );
        }
        
        /**
         */
        void Set( f32 w, f32 x, f32 y, f32 z )
        {
             W() = w; X() = x; Y() = y; Z() = z;
        }

        /**
         */
        void SetAll( f32 v )
        {
            Set( v, v, v, v );
        }
        
        /** 3ݒ肷
         */
        void SetFromAxises( const Vec3F_st & cr_Xa, const Vec3F_st & cr_Ya, const Vec3F_st & cr_Za )
        {
            f32 tr = cr_Xa.X() + cr_Ya.Y() + cr_Za.Z();
            f32 s;
            
            if ( tr > 0.f )
            {
                s = Sqrt( tr + 1.f );
                W() = s * 0.5f;
                s   = 0.5f / s;
                X() = ( cr_Ya.Z() - cr_Za.Y() ) * s;
                Y() = ( cr_Za.X() - cr_Xa.Z() ) * s;
                Z() = ( cr_Xa.Y() - cr_Ya.X() ) * s;
            }
            else
            {
                int i = 0;
                if ( cr_Ya.Y() > cr_Xa.X() )
                {
                    i = 1;
                    if ( cr_Za.Z() > cr_Ya.Y() ) i = 2;
                }
                else
                {
                    if ( cr_Za.Z() > cr_Xa.X() ) i = 2;
                }

                switch ( i )
                {
                case 0: //  x
                    s   = Sqrt( ( cr_Xa.X() - ( cr_Ya.Y() + cr_Za.Z() ) ) + 1.f );
                    X() = s * 0.5f;
                    s = 0.5f / s;
                    Y() = ( cr_Xa.Y() + cr_Ya.X() ) * s;
                    Z() = ( cr_Za.X() + cr_Xa.Z() ) * s;
                    W() = ( cr_Ya.Z() - cr_Za.Y() ) * s;
                    break;
                case 1: //  y
                    s   = Sqrt( ( cr_Ya.Y() - ( cr_Za.Z() + cr_Xa.X() ) ) + 1.f );
                    Y() = s * 0.5f;
                    s = 0.5f / s;
                    X() = ( cr_Xa.Y() + cr_Ya.X() ) * s;
                    Z() = ( cr_Ya.Z() + cr_Za.Y() ) * s;
                    W() = ( cr_Za.X() - cr_Xa.Z() ) * s;
                    break;
                case 2: //  z
                    s   = Sqrt( ( cr_Za.Z() - ( cr_Xa.X() + cr_Ya.Y() ) ) + 1.f );
                    Z() = s * 0.5f;
                    s = 0.5f / s;
                    X() = ( cr_Za.X() + cr_Xa.Z() ) * s;
                    Y() = ( cr_Ya.Z() + cr_Za.Y() ) * s;
                    W() = ( cr_Xa.Y() - cr_Ya.X() ) * s;
                    break;
                }
            }
        }

        /** 3ݒ肷
         */
        void GetAxises( Vec3F_st & r_Xa, Vec3F_st & r_Ya, Vec3F_st & r_Za ) const
        {
            f32 s,xs,ys,zs,wx,wy,wz,xx,xy,xz,yy,yz,zz;
            s = 2.f / Dot( *this );

            xs = X() *  s;     ys = Y() *  s;  zs = Z() *  s;
            wx = W() * xs;     wy = W() * ys;  wz = W() * zs;
            xx = X() * xs;     xy = X() * ys;  xz = X() * zs;
            yy = Y() * ys;     yz = Y() * zs;  zz = Z() * zs;

            r_Xa.Set( 1.f - ( yy + zz ), xy + wz, xz - wy );
            r_Ya.Set( xy - wz, 1.f - ( xx + zz ), yz + wx );
            r_Za.Set( xz + wy, yz - wx, 1.f - ( xx + yy ) );
        }

        /** 
         */
        f32 Dot( const Quat_st & cr ) const
        {
            return X() * cr.X() + Y() * cr.Y() + Z() * cr.Z() + W() * cr.W();
        }

        /**
         */
        void Scale( f32 scale )
        {
            W() *= scale;
            X() *= scale;
            Y() *= scale;
            Z() *= scale;
        }


        /**
         */
        f32 SquareNorm() const
        {
            return Dot( *this );
        }

        
        /** K
         */
        void Normalize()
        {
            f32 mag = SquareNorm();

            if ( mag >= c_Epsilon )
            {
                mag = 1.f / Sqrt( mag );

                Scale( mag );
            }
            else
            {
                SetAll( 0.f );
            }
        }
        
        /** ʐ`
         *  @param cr_P
         *  @param cr_Q
         *  @param t    ԌW(0.0:cr_P - 1.0:cr_Q)
         */
        void Slerp( const Quat_st & cr_P, const Quat_st & cr_Q, f32 t )
        {
            f32 cosTh = cr_P.Dot( cr_Q );
            f32 tp;
            f32 tq = 1.f;
            
            if ( cosTh < 0.f )
            {
                cosTh = -cosTh;
                tq    = -tq;
            }
            
            if ( cosTh <= 1.f - c_Epsilon )
            {
                f32 theta = ACos( cosTh );
                f32 sinTh = Sin( theta );
                tp  = Sin( ( 1.f - t ) * theta ) / sinTh;
                tq *= Sin( t * theta ) / sinTh;
            }
            else
            {
                tp  = 1.f - t;
                tq *= t;
            }

            W() = tp * cr_P.W() + tq * cr_Q.W();
            X() = tp * cr_P.X() + tq * cr_Q.X();
            Y() = tp * cr_P.Y() + tq * cr_Q.Y();
            Z() = tp * cr_P.Z() + tq * cr_Q.Z();
        }

        /** ]擾
         *  @pre    PʃNH[^jIłKv
         *  @param[out] r_Vec   ]
         *  @retval             ]p(rad)
         */
        f32 GetRotate( Vec3F_st & r_Vec )
        {
            f32 theta_2     = ACos( W() );
            f32 sinTheta_2  = Sin( theta_2 );

            r_Vec.Set( X(), Y(), Z() );
            if ( sinTheta_2 != 0.f )
                r_Vec.Div( sinTheta_2 );

            return 2.f * theta_2;
        }

        /** 
         */
        void Mult( const Quat_st & cr )
        {
            Set( W() * cr.W() - X() * cr.X() - Y() * cr.Y() - Z() * cr.Z(),
                 W() * cr.X() + X() * cr.W() + Y() * cr.Z() - Z() * cr.Y(),
                 W() * cr.Y() + Y() * cr.W() + Z() * cr.X() - X() * cr.Z(),
                 W() * cr.Z() + Z() * cr.W() + X() * cr.Y() - Y() * cr.X() );
        }

        /** 
         */
        void SetConjugate( const Quat_st & cr )
        {
            Set( cr.W(), -cr.X(), -cr.Y(), -cr.Z() );
        }

        /** tNH[^jI
         */
        void SetInverse( const Quat_st & cr )
        {
            f32 sqNorm = cr.SquareNorm();
            if ( sqNorm > c_Epsilon )
            {
                f32 sqNormInv = 1.f / sqNorm;
                W() =   cr.W() * sqNormInv;
                X() = - cr.X() * sqNormInv;
                Y() = - cr.Y() * sqNormInv;
                Z() = - cr.Z() * sqNormInv;
            }
            else
            {
                SetConjugate( cr );
            }
        }
        
        f32 & W() { return ma_[ 0 ]; }
        f32 & X() { return ma_[ 1 ]; }
        f32 & Y() { return ma_[ 2 ]; }
        f32 & Z() { return ma_[ 3 ]; }
        const f32 & W() const { return ma_[ 0 ]; }
        const f32 & X() const { return ma_[ 1 ]; }
        const f32 & Y() const { return ma_[ 2 ]; }
        const f32 & Z() const { return ma_[ 3 ]; }

        f32 ma_[ 4 ];
    };
}
}
#endif
/// @endcond
